@@ -1,4 +1,9 @@
 from __future__ import annotations
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from parchmint.device import Device
+
 from typing import List, Tuple
 from parchmint.params import Params
 from parchmint.port import Port
@@ -20,8 +25,6 @@ class Component:
         self.ID: str = ""
         self.params = Params()
         self.entity: str = ""
-        self.xpos: int = -1
-        self.ypos: int = -1
         self.xspan: int = -1
         self.yspan: int = -1
         self.ports: List[Port] = []
@@ -34,6 +37,66 @@ class Component:
                 )
             self.parse_from_json(json, device_ref)
 
+    @property
+    def xpos(self) -> int:
+        """returns the x coordinate of the component
+
+        Raises:
+            KeyError: when no position parameter object is found for the parchmint object
+
+        Returns:
+            int: x-coordinate
+        """
+        try:
+            return self.params.get_param("position")[0]
+        except Exception:
+            print("Could not find xpos for component")
+            raise KeyError
+
+    @xpos.setter
+    def xpos(self, value) -> None:
+        """Sets the x-coordinate for the component
+
+        Args:
+            value (int): x coordianate of the object
+        """
+        if self.params.exists("position"):
+            pos = self.params.get_param("position")
+            pos[0] = value
+            self.params.set_param("position", pos)
+        else:
+            self.params.set_param("position", [value, -1])
+
+    @property
+    def ypos(self) -> int:
+        """Returns the y-coordinate in the parchmint object
+
+        Raises:
+            KeyError: When no position parameter is found in the parchmint object
+
+        Returns:
+            int: y coordinate of the component
+        """
+        try:
+            return self.params.get_param("position")[1]
+        except Exception:
+            print("Could not find xpos for component")
+            raise KeyError
+
+    @ypos.setter
+    def ypos(self, value) -> None:
+        """Sets the y-coordinate of the component
+
+        Args:
+            value (int): y coordinate
+        """
+        if self.params.exists("position"):
+            pos = self.params.get_param("position")
+            pos[1] = value
+            self.params.set_param("position", pos)
+        else:
+            self.params.set_param("position", [-1, value])
+
     def add_component_ports(self, ports: List[Port]) -> None:
         """Adds component ports to the component
 
@@ -81,7 +144,8 @@ class Component:
         Returns:
             dict: dictionary that can be used in json.dumps()
         """
-        return {
+
+        ret = {
             "name": self.name,
             "id": self.ID,
             "layers": [layer.ID for layer in self.layers],
@@ -92,6 +156,8 @@ class Component:
             "y-span": int(self.yspan),
         }
 
+        return ret
+
     def __eq__(self, obj):
         if isinstance(obj, Component):
             return obj.ID == self.ID
