@@ -5,6 +5,67 @@ from parchmint.params import Params
 from parchmint.target import Target
 
 
+class ConnectionPath:
+    def __init__(
+        self,
+        source: Target = None,
+        sink: Target = None,
+        waypoints: List[Tuple[int, int]] = [],
+        json=None,
+    ) -> None:
+        """Creates a new connection path object
+
+        Args:
+            source (Target): source corresponding to the path
+            sink (Target): sink corresponding to the path
+            waypoints (List[Tuple[int, int]], optional): list of the coordinates. Defaults to [].
+        """
+        super().__init__()
+        self.__source: Optional[Target] = source
+        self.__sink: Optional[Target] = sink
+        self.__waypoints: List[Tuple[int, int]] = waypoints
+
+        if json is not None:
+            self.parse_parchmint_v1(json)
+
+    @property
+    def source(self) -> Target:
+        assert self.__source is not None
+        return self.__source
+
+    @property
+    def sink(self) -> Target:
+        assert self.__sink is not None
+        return self.__sink
+
+    @property
+    def waypoints(self) -> List[Tuple[int, int]]:
+        return self.__waypoints
+
+    @waypoints.setter
+    def waypoints(self, value: List[Tuple[int, int]]):
+        self.__waypoints = value
+
+    def to_parchmint_v1(self):
+        """Returns the json dict
+
+        Returns:
+            dict: dictionary that can be used in json.dumps()
+        """
+        return {
+            "source": None
+            if self.__source is None
+            else self.__source.to_parchmint_v1(),
+            "sink": None if self.__sink is None else self.__sink.to_parchmint_v1(),
+            "wayPoints": [list(wp) for wp in self.__waypoints],
+        }
+
+    def parse_parchmint_v1(self, json) -> None:
+        self.__source = Target(json=json["source"])
+        self.__sink = Target(json=json["sink"])
+        self.__waypoints = [(wp[0], wp[1]) for wp in json["wayPoints"]]
+
+
 class Connection:
     def __init__(self, json=None, device_ref=None):
         """[summary]
@@ -22,7 +83,8 @@ class Connection:
         self.params: Params = Params()
         self.source: Optional[Target] = None
         self.sinks: List[Target] = []
-        self.layer: Layer = None
+        self.layer: Optional[Layer] = None
+        self._paths: List[ConnectionPath] = []
 
         if json:
             if device_ref is None:
@@ -46,12 +108,31 @@ class Connection:
         self.name = json["name"]
         self.ID = json["id"]
         self.layer = device_ref.get_layer(json["layer"])
+
+        # Pull out the paths
+        if "paths" in json["params"].keys():
+            json_paths = json["params"]["paths"]
+            for json_path in json_paths:
+                self._paths.append(ConnectionPath(json_path))
+
         self.params = Params(json["params"])
 
         self.source = Target(json["source"])
 
-        for target in json["sinks"]:
-            self.sinks.append(Target(target))
+        if "sinks" in json.keys():
+            if json["sinks"]:
+                for target in json["sinks"]:
+                    self.sinks.append(Target(target))
+            else:
+                print("connection", self.name, "does not have any sinks")
+        else:
+            print("connection", self.name, "does not have any sinks")
+
+        # TODO - Change this in the v1.2 version
+        if "waypoints" in json.keys():
+            waypoints_raw = json["waypoints"]
+            waypoints = [(wp[0], wp[1]) for wp in waypoints_raw]
+            self.add_waypoints_path(None, None, waypoints)
 
     def __str__(self):
         return str(self.__dict__)
@@ -59,6 +140,10 @@ class Connection:
     def __repr__(self):
         return str(self.__dict__)
 
+    @property
+    def paths(self) -> List[ConnectionPath]:
+        return self._paths
+
     def add_waypoints_path(
         self, source: Target, sink: Target, waypoints: List[Tuple[int, int]]
     ) -> None:
@@ -70,11 +155,7 @@ class Connection:
             waypoints (List[Tuple[int, int]]): array of coordinates (as tuples)
         """
         path = ConnectionPath(source, sink, waypoints)
-        if self.params.exists("paths"):
-            paths = self.params.get_param("paths")
-            paths.append(path.to_parchmint_v1())
-        else:
-            self.params.set_param("paths", [path.to_parchmint_v1()])
+        self._paths.append(path)
 
     def to_parchmint_v1(self):
         """Returns the json dict
@@ -82,7 +163,7 @@ class Connection:
         Returns:
             dict: dictionary that can be used in json.dumps()
         """
-        return {
+        ret = {
             "sinks": [s.to_parchmint_v1() for s in self.sinks],
             "name": self.name,
             "id": self.ID,
@@ -91,33 +172,6 @@ class Connection:
             "layer": self.layer.ID,
         }
 
+        ret["paths"] = [path.to_parchmint_v1() for path in self._paths]
 
-class ConnectionPath:
-    def __init__(
-        self, source: Target, sink: Target, waypoints: List[Tuple[int, int]] = []
-    ) -> None:
-        """Creates a new connection path object
-
-        Args:
-            source (Target): source corresponding to the path
-            sink (Target): sink corresponding to the path
-            waypoints (List[Tuple[int, int]], optional): list of the coordinates. Defaults to [].
-        """
-        super().__init__()
-        self.__source: Target = source
-        self.__sink: Target = sink
-        self.__waypoints: List[Tuple[int, int]] = waypoints
-
-    def to_parchmint_v1(self):
-        """Returns the json dict
-
-        Returns:
-            dict: dictionary that can be used in json.dumps()
-        """
-        return {
-            "source": None
-            if self.__source is None
-            else self.__source.to_parchmint_v1(),
-            "sinks": None if self.__sink is None else self.__sink.to_parchmint_v1(),
-            "wayPoints": [list(wp) for wp in self.__waypoints],
-        }
+        return ret
