@@ -1,17 +1,50 @@
 from __future__ import annotations
+
 from parchmint.layer import Layer
 import networkx as nx
-from typing import Optional, List
+from typing import Dict, Optional, List
 from parchmint.component import Component
 from parchmint.connection import Connection
 from parchmint.params import Params
+from parchmint.similaritymatcher import SimilarityMatcher
+
 import jsonschema
 import pathlib
 import parchmint
 import json
+from enum import Enum
+
 
 PROJECT_DIR = pathlib.Path(parchmint.__file__).parent.parent.absolute()
 
+# GM = nx.algorithms.isomorphism.GraphMatcher(device1, device2)
+
+
+class ValveType(Enum):
+    NORMALLY_OPEN = 0
+    NORMALLY_CLOSED = 1
+
+    def __str__(self) -> str:
+        if self == ValveType.NORMALLY_OPEN:
+            return "NORMALLY_OPEN"
+        elif self == ValveType.NORMALLY_CLOSED:
+            return "NORMALLY_CLOSED"
+        else:
+            raise Exception("Could not generate Valve Type string")
+
+    def __eq__(self, o: object) -> bool:
+        if o.__class__ is ValveType:
+            return super().__eq__(o)
+        elif o.__class__ is str:
+            if self is ValveType.NORMALLY_OPEN and o == "NORMALLY_OPEN":
+                return True
+            elif self is ValveType.NORMALLY_CLOSED and o == "NORMALLY_CLOSED":
+                return True
+            else:
+                return False
+        else:
+            return False
+
 
 class Device:
     def __init__(self, json=None):
@@ -30,10 +63,103 @@ class Device:
         self.yspan: Optional[int] = None
         self.G = nx.MultiDiGraph()
 
+        # Stores the valve / connection mappings
+        self._valve_map: Dict[Component, Connection] = dict()
+        self._valve_type_map: Dict[Component, ValveType] = dict()
+
         if json:
             self.parse_from_json(json)
             self.generate_network()
 
+    def map_valve(
+        self,
+        valve: Component,
+        connection: Connection,
+        type_info: Optional[ValveType] = None,
+    ) -> None:
+        """Maps the valve to a connection in the device
+
+        Args:
+            valve (Component): valve component
+            connection (Connection): connection on which the valve is mapped
+            type_info (Optional[ValveType]): Type informaiton of the valve
+
+        """
+        self._valve_map[valve] = connection
+        if type_info is not None:
+            self.update_valve_type(valve, type_info)
+
+    def get_valves(self) -> List[Component]:
+        """Returns the list of valves in the device
+
+        Returns:
+            List[Component]: Valve Component Objects
+        """
+        return list(self._valve_map.keys())
+
+    def get_valve_connection(self, valve: Component) -> Connection:
+        """Returns the connection associated with the valve object
+
+        Args:
+            valve (Component): Valve object for which we are finding the connection
+
+        Returns:
+            Connection: connection object on which the valve is placed
+        """
+        return self._valve_map[valve]
+
+    def update_valve_type(self, valve: Component, type_info: ValveType) -> None:
+        """Updates the type of the valve to normally closed  or normally open
+
+        Args:
+            valve (Component): Valve object we want to update
+            type_info (ValveType): Valve Type
+
+        Raises:
+            KeyError: Raises the error if the valve object is not mapped as a valve in the device
+        """
+        if valve in list(self._valve_map.keys()):
+            self._valve_type_map[valve] = type_info
+        else:
+            raise KeyError(
+                "Could not update type for valve: {} since it is not found in the valveMap of they device".format(
+                    valve.ID
+                )
+            )
+
+    # add compare function
+    # -pass device, compare devices
+    # - check connections, components, print if its same or not
+    # - have a flag to print parameter differences
+
+    def compare(self, device: Device) -> bool:
+        """compare against the input device. Return true if they are semnatcally feasible.
+
+        Args:
+            device (Device): expected device
+
+        Returns:
+            bool: If semntically feasible, return true. Else false.
+        """
+
+        self.generate_network()
+
+        SM = SimilarityMatcher(self, device)
+
+        is_same = SM.is_isomorphic()
+        SM.print_params_diff()
+        SM.print_layers_diff()
+        SM.print_port_diff()
+        SM.print_in_edges_diff()
+        SM.print_out_edges_diff()
+
+        if is_same:
+            print("Match!")
+        else:
+            print("Not Match!")
+
+        return is_same
+
     def add_component(self, component: Component):
         """Adds a component object to the device
 
@@ -45,7 +171,7 @@ class Device:
         """
         if isinstance(component, Component):
             # Check if Component Exists, if it does ignore it
-            if self.does_component_exist(component):
+            if self.component_exists(component.ID):
                 print(
                     "Component {} already present in device, "
                     "hence skipping the component".format(component.name)
@@ -113,6 +239,7 @@ class Device:
                 self.add_layer(layer)
                 layer_mapping[layer] = layer
             else:
+                assert layer.ID is not None
                 layer_mapping[layer] = self.get_layer(layer.ID)
 
         for component in netlist.components:
@@ -135,15 +262,24 @@ class Device:
         self.name = json["name"]
 
         # First always add the layers
-        for layer in json["layers"]:
-            self.add_layer(Layer(layer))
+        if "layers" in json.keys():
+            for layer in json["layers"]:
+                self.add_layer(Layer(layer))
+        else:
+            print("no layers found")
 
         # Loop through the components
-        for component in json["components"]:
-            self.add_component(Component(component, self))
+        if "components" in json.keys():
+            for component in json["components"]:
+                self.add_component(Component(component, self))
+        else:
+            print("no components found")
 
-        for connection in json["connections"]:
-            self.add_connection(Connection(connection, self))
+        if "connections" in json.keys():
+            for connection in json["connections"]:
+                self.add_connection(Connection(connection, self))
+        else:
+            print("no connections found")
 
         if "params" in json.keys():
             self.params = Params(json["params"])
@@ -161,6 +297,27 @@ class Device:
                 self.yspan = self.params.get_param("length")
             elif self.params.exists("y-span"):
                 self.yspan = self.params.get_param("y-span")
+        else:
+            print("no params found")
+        
+        if "valveMap" in json.keys():
+            valve_map = json["valveMap"]
+
+            for key, value in valve_map.items():
+                self._valve_map[self.get_component(key)] = self.get_connection(value)
+
+        if "valveTypeMap" in json.keys():
+            valve_type_map = json["valveTypeMap"]
+
+            for key, value in valve_type_map.items():
+                if value is ValveType.NORMALLY_OPEN:
+                    self._valve_type_map[
+                        self.get_component(key)
+                    ] = ValveType.NORMALLY_OPEN
+                else:
+                    self._valve_type_map[
+                        self.get_component(key)
+                    ] = ValveType.NORMALLY_CLOSED
 
     def get_components(self) -> List[Component]:
         """Returns the components in the device
@@ -294,6 +451,23 @@ class Device:
 
         return ret
 
+    def to_parchmint_v1_x(self):
+        ret = self.to_parchmint_v1()
+
+        # Modify the version of the parchmint
+        ret["version"] = 1.2
+
+        # Add the valvemap information
+        valve_map = dict()
+        valve_type_map = dict()
+        for valve, connection in self._valve_map.items():
+            valve_map[valve.ID] = connection.ID
+        ret["valveMap"] = valve_map
+        for valve, valve_type in self._valve_type_map.items():
+            valve_type_map[valve.ID] = str(valve_type)
+        ret["valveTypeMap"] = valve_type_map
+        return ret
+
     @staticmethod
     def validate_V1(json_str: str) -> None:
         """Validates the json string against the schema
